<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Geeklog 1.3                                                               |
// +---------------------------------------------------------------------------+
// | atom.feed.class.php                                                       |
// |                                                                           |
// | Geeklog class for Atom feeds.                                         |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2005 by the following authors:                         |
// |                                                                           |
// | Authors: Tony Bibbs       - tony AT geeklog DOT net                       |
// |          Dirk Haun        - dirk AT haun-online DOT de                    |
// |		  Michael Jervis   - mike AT fuckingbrit DOT com                   |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
// $Id: atom.feed.class.php,v 1.3 2004/04/10 11:43:26 themike Exp $

/**
* Geeklog Atom class
*
* Parts of this code have originally been lifted from phpweblog which is
* licenced under the GPL. It has since been heavily modified.
*
* @author Dirk Haun <dirk AT haun-online DOT de>
* @package net.geeklog.rss
*
*/
class Atom {
    /**
    * @access private
    * @var string
    */
    var $_feedfile = 'geeklog.xml';

    /**
    * @access private
    * @var string
    */
    var $_feedpath = '';

    /**
    * @access private
    * @var string
    */
    var $_feedurl = '';

    /**
    * @access private
    * @var string
    */
    var $_sitelink = '';

    /**
    * @access private
    * @var string
    */
    var $_feedtitle = '';

    /**
    * @access private
    * @var string
    */
    var $_feeddesc = '';

    /**
    * @access private
    * @var string
    */
    var $_feedlang = 'en-gb';

    /**
    * @access private
    * @var string
    */
    var $_feedencoding = 'UTF-8';

    /**
    * @access private
    * @var int
    */
    var $_feedcontentlen = 0;


    /**
    * Constructor
    *
    * Initializes private variables.
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function Rdf ()
    {
        global $_CONF;

        if (!empty ($_CONF['default_charset'])) {
            $this->_feedencoding = $_CONF['default_charset'];
        } else {
            $this->_feedencoding = 'UTF-8';
        }

        if (!empty ($_CONF['rdf_language'])) {
            $this->_feedlang = $_CONF['rdf_language'];
        } else {
            $this->_feedlang = $_CONF['locale'];
        }
    }

    /**
    * Set file name for the Atom feed.
    *
    * @param   string   filename   actual file name, e.g. 'geeklog.rdf'
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function setFeedfile ($filename = '')
    {
        global $_CONF;

        if (!empty ($filename)) {
            $this->_feedfile = $filename;
        } else {
            $pos = strrpos ($_CONF['rdf_file'], '/');
            $this->_feedfile = substr ($_CONF['rdf_file'], $pos + 1);
        }
        $path = $_CONF['rdf_file'];
        $pos = strrpos ($path, '/');
        $path = substr ($path, 0, $pos + 1);
        $this->_feedpath = $path . $this->_feedfile;
        $this->_feedurl = substr_replace ($path, $_CONF['site_url'], 0,
                                          strlen ($_CONF['path_html']) - 1);
    }

    /**
    * Set information for an Atom feed.
    *
    * @param string   link    link to the site where this feed originates
    * @param string   title   (short) title of the feed
    * @param string   desc    (longer) description of the feed
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function setFeedinfo ($link = '', $title = '', $desc = '')
    {
        $this->_sitelink = $link;
        $this->_feedtitle = $title;
        $this->_feeddesc = $desc;
    }

    /**
    * Set formats for an Atom feed.
    *
    * @param int      contentlen    length of the content of an entry
    * @param string   language      feed language, e.g. 'en-gb'
    * @param string   encoding      feed encoding, e.g. 'UTF-8'
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function setFeedformats ($contentlen = 0, $language = '', $encoding = '')
    {
        $this->_feedcontentlen = $contentlen;

        if (!empty ($language)) {
            $this->_feedlang = $language;
        }

        if (!empty ($encoding)) {
            $this->_feedencoding = $encoding;
        }
    }

    /**
    * Format the content of an item
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access private
    *
    */
    function _formatContent ($text)
    {
        $storytext = trim (strip_tags ($text));
        $storytext = preg_replace ("/(\015)/", "", $storytext);
        if ($this->_feedcontentlen > 1) {
            if (strlen ($storytext) > $this->_feedcontentlen) {
                $storytext = substr ($storytext, 0, $this->_feedcontentlen - 3)
                           . '...';
            }
        }

        return htmlspecialchars ($storytext);
    }
    
    /**
    * Turns a UID into an author name
    *
    * @author Michael Jervis (mike AT fuckingbrit DOT com)
    * @access private
    */
    function _uid2author( $uid )
    {
    	global $_TABLES;
    	
    	return DB_GetItem( $_TABLES['users'], 'username', "uid='$uid'" ); 
    }

    /**
    * Write the actual feed file.
    *
    * @author Michael Jervis (mike AT fuckingbrit DOT com)
    * @access public
    *
    */
    function write ($content)
    {
        global $LANG01, $_CONF;

        if (sizeof ($content) == 0) {
            return false;
        }

        $success = false;

        if ($fd = @fopen ($this->_feedpath, 'w')) {
            fputs( $fd, "<?xml version=\"1.0\" encoding=\"$this->_feedencoding\"?>\n\n" );
            fputs( $fd, "<feed version=\"0.3\" xmlns=\"http://purl.org/atom/ns#\">\n" );
            fputs( $fd, '<title mode="escaped">'.htmlspecialchars( $this->_feedtitle )."</title>\n" );
            fputs( $fd, '<tagline mode="escaped">'.htmlspecialchars( $this->_feeddesc )."</tagline>\n" );
            fputs( $fd, '<link rel="alternate" type="text/html" href="'.htmlspecialchars( $this->_sitelink )."\"/>\n" );
            fputs( $fd, '<modified>'.strftime( "%Y-%m-%dT%TZ" )."</modified>\n" );
            
            foreach ($content as $entry) {
                $content = '';
                if ($this->_feedcontentlen > 0) {
                    $content = '<content type="text/html" mode="escaped">'
                          . $this->_formatContent ($entry['text'])
                          . "</content>\n";
                }
                $title = '<title mode="escaped">' . htmlspecialchars ($entry['title'])
                       . "</title>\n";
                $link = '<link rel="alternate" type="text/html" href="' . htmlspecialchars ($entry['link'])
                      . "\"/>\n";
                $id = '<id>'.htmlspecialchars( $entry['link'] )."</id>\n";
                
                if( empty( $entry['author'] ) )
                {
                	$author = "<author>\n<name>".htmlspecialchars( $this->_uid2author( $entry['uid'] ) )."</name>\n</author>\n";
                } else {
                	$author = "<author>\n<name>".htmlspecialchars( $entry['author'] )."</name>\n</author>\n";
               	}

				$issued = '<issued>'.htmlspecialchars( strftime( "%Y-%m-%dT%TZ", $entry['date'] ) )."</issued>";
				$modified = '<modified>'.htmlspecialchars( strftime( "%Y-%m-%dT%TZ", $entry['date'] ) )."</modified>";
				fputs( $fd, "<entry>\n" );
				fputs( $fd, $title );
				fputs( $fd, $id );
				fputs( $fd, $link );
				fputs( $fd, $issued );
				fputs( $fd, $modified );
				fputs( $fd, $author );
				fputs( $fd, $content );
                fputs( $fd, "</entry>\n\n");
            }

            fputs ($fd, "</feed>\n");

            fclose ($fd);

            $success = true;
        } else {
            COM_errorLog ($LANG01[54] . ' ' . $this->_feedpath, 1);
        }

        return $success;
    }
}

?>
