<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Geeklog 1.3                                                               |
// +---------------------------------------------------------------------------+
// | rss2.feed.class.php                                                        |
// |                                                                           |
// | Geeklog class for RSS 2.0 feeds.                                         |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2005 by the following authors:                         |
// |                                                                           |
// | Authors: Tony Bibbs       - tony AT geeklog DOT net                       |
// |          Dirk Haun        - dirk AT haun-online DOT de                    |
// |		  Michael Jervis   - mike AT fuckingbrit DOT com                   |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+
// $Id: rss2.feed.class.php,v 1.1 2004/03/19 08:47:21 themike Exp $

/**
* Geeklog RSS 2.0 class
*
* Parts of this code have originally been lifted from phpweblog which is
* licenced under the GPL. It has since been heavily modified.
*
* @author Dirk Haun <dirk AT haun-online DOT de>, Michael Jervis <mike AT fuckingbrit DOT com>
* @package net.geeklog.rss
*
*/
class Rss2 {
    /**
    * @access private
    * @var string
    */
    var $_feedfile = 'geeklog.rdf';

    /**
    * @access private
    * @var string
    */
    var $_feedpath = '';

    /**
    * @access private
    * @var string
    */
    var $_feedurl = '';

    /**
    * @access private
    * @var string
    */
    var $_sitelink = '';

    /**
    * @access private
    * @var string
    */
    var $_feedtitle = '';

    /**
    * @access private
    * @var string
    */
    var $_feeddesc = '';

    /**
    * @access private
    * @var string
    */
    var $_feedlang = 'en-gb';

    /**
    * @access private
    * @var string
    */
    var $_feedencoding = 'UTF-8';

    /**
    * @access private
    * @var int
    */
    var $_feedcontentlen = 0;


    /**
    * Constructor
    *
    * Initializes private variables.
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function Rss2 ()
    {
        global $_CONF;

        if (!empty ($_CONF['default_charset'])) {
            $this->_feedencoding = $_CONF['default_charset'];
        } else {
            $this->_feedencoding = 'UTF-8';
        }

        if (!empty ($_CONF['rdf_language'])) {
            $this->_feedlang = $_CONF['rdf_language'];
        } else {
            $this->_feedlang = $_CONF['locale'];
        }
    }

    /**
    * Set file name for the RSS feed.
    *
    * @param   string   filename   actual file name, e.g. 'geeklog.rdf'
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function setFeedfile ($filename = '')
    {
        global $_CONF;

        if (!empty ($filename)) {
            $this->_feedfile = $filename;
        } else {
            $pos = strrpos ($_CONF['rdf_file'], '/');
            $this->_feedfile = substr ($_CONF['rdf_file'], $pos + 1);
        }
        $path = $_CONF['rdf_file'];
        $pos = strrpos ($path, '/');
        $path = substr ($path, 0, $pos + 1);
        $this->_feedpath = $path . $this->_feedfile;
        $this->_feedurl = substr_replace ($path, $_CONF['site_url'], 0,
                                          strlen ($_CONF['path_html']) - 1);
    }

    /**
    * Set information for an RSS feed.
    *
    * @param string   link    link to the site where this feed originates
    * @param string   title   (short) title of the feed
    * @param string   desc    (longer) description of the feed
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function setFeedinfo ($link = '', $title = '', $desc = '')
    {
        $this->_sitelink = $link;
        $this->_feedtitle = $title;
        $this->_feeddesc = $desc;
    }

    /**
    * Set formats for an RSS feed.
    *
    * @param int      contentlen    length of the content of an entry
    * @param string   language      feed language, e.g. 'en-gb'
    * @param string   encoding      feed encoding, e.g. 'UTF-8'
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access public
    *
    */
    function setFeedformats ($contentlen = 0, $language = '', $encoding = '')
    {
        $this->_feedcontentlen = $contentlen;

        if (!empty ($language)) {
            $this->_feedlang = $language;
        }

        if (!empty ($encoding)) {
            $this->_feedencoding = $encoding;
        }
    }

    /**
    * Format the content of an item
    *
    * @author Dirk Haun <dirk AT haun-online DOT de>
    * @access private
    *
    */
    function _formatContent ($text)
    {
        $storytext = trim (strip_tags ($text));
        $storytext = preg_replace ("/(\015)/", "", $storytext);
        if ($this->_feedcontentlen > 1) {
            if (strlen ($storytext) > $this->_feedcontentlen) {
                $storytext = substr ($storytext, 0, $this->_feedcontentlen - 3)
                           . '...';
            }
        }

        return htmlspecialchars ($storytext);
    }
    
    /**
    * Turns a UID into an author name
    *
    * @author Michael Jervis (mike AT fuckingbrit DOT com)
    * @access private
    */
    function _uid2author( $uid )
    {
    	global $_TABLES;
    	
    	return DB_GetItem( $_TABLES['users'], 'username', "uid='$uid'" ); 
    }

    /**
    * Write the actual feed file.
    *
    * @author Michael Jervis (mike AT fuckingbrit DOT com)
    * @access public
    *
    */
    function write ($content)
    {
        global $LANG01, $_CONF;

        if (sizeof ($content) == 0) {
            return false;
        }

        $success = false;

        if ($fd = @fopen ($this->_feedpath, 'w')) {
        	fputs( $fd, "<?xml version=\"1.0\"?>\n\n" );
        	fputs( $fd, "<rss version=\"2.0\">\n" );
        	fputs( $fd, "<channel>\n" );
        	/* Required RSS 2.0 elements for channel: */
        	fputs( $fd, '<title><![CDATA['.htmlspecialchars( $this->_feedtitle )."]]></title>\n" );
        	fputs( $fd, '<link>'.htmlspecialchars( $this->_sitelink )."</link>\n" );
        	fputs( $fd, '<description><![CDATA['.htmlspecialchars( $this->_feeddesc )."]]></description>\n" );
        	
        	/* Optional RSS 2.0 elements for channel: */
        	fputs( $fd, '<managingEditor>'.htmlspecialchars( $_CONF['site_mail'] )."</managingEditor>\n" );
        	fputs( $fd, '<webMaster>'.htmlspecialchars( $_CONF['site_mail'] )."</webMaster>\n" );
        	fputs( $fd, '<language>'.htmlspecialchars( $this->_feedlang )."</language>\n" );
        	fputs( $fd, '<copyright><![CDATA['.htmlspecialchars( 'Copyright '.strftime( "%Y" ).' '.$_CONF['site_name'] )."]]></copyright>\n" );
        	fputs( $fd, '<pubDate>'.strftime( "%a, %d %b %Y %T %Z" )."</pubDate>\n" );
        	fputs( $fd, '<generator>Geeklog '.VERSION."</generator>\n" );
        	
            foreach ($content as $entry) {
                $desc = '';
                if ($this->_feedcontentlen > 0) {
                    $desc = '<description><![CDATA['
                          . $this->_formatContent ($entry['text'])
                          . "]]></description>\n";
                }
                $title = '<title><![CDATA[' . htmlspecialchars ($entry['title'])
                       . "]]></title>\n";
                $link = '<link>' . htmlspecialchars ($entry['link'])
                      . "</link>\n";

                fputs( $fd, "<item>\n" );
                fputs( $fd, '<guid isPermaLink="true">'.htmlspecialchars( $entry['link'] )."</guid>\n" );
                fputs( $fd, $link );
                fputs( $fd, '<pubDate>'.htmlspecialchars( strftime( "%a, %d %b %Y %T %Z", $entry['date'] ) )."</pubDate>" );
                fputs( $fd, $title );
                fputs( $fd, '<comments>'.htmlspecialchars( $entry['link'].'#comments' )."</comments>\n" );
                
                if (!empty ($desc)) {
                    fputs ($fd, $desc);
                }
                fputs ($fd, "</item>\n\n");
            }

            fputs ($fd, "</channel>\n</rss>");

            fclose ($fd);

            $success = true;
        } else {
            COM_errorLog ($LANG01[54] . ' ' . $this->_feedpath, 1);
        }

        return $success;
    }
}

?>
